@@ -20,14 +20,14 @@ namespace newarp {
 template <typename eT>
 inline DenseGenMatProd<eT>::DenseGenMatProd(const Mat<eT>& mat_obj)
     : op_mat(mat_obj), n_rows(mat_obj.n_rows), n_cols(mat_obj.n_cols) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
 }
 
 // Perform the matrix-vector multiplication operation \f$y=Ax\f$.
 // y_out = A * x_in
 template <typename eT>
 inline void DenseGenMatProd<eT>::perform_op(eT* x_in, eT* y_out) const {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
 
   const Col<eT> x(x_in, n_cols, false, true);
   Col<eT> y(y_out, n_rows, false, true);
