@@ -20,12 +20,12 @@
 
 template <typename T1>
 inline void op_max::apply(Mat<typename T1::elem_type>& out, const Op<T1, op_max>& in) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
 
   typedef typename T1::elem_type eT;
 
   const uword dim = in.aux_uword_a;
-  arma_debug_check((dim > 1), "max(): parameter 'dim' must be 0 or 1");
+  arma_conform_check((dim > 1), "max(): parameter 'dim' must be 0 or 1");
 
   const quasi_unwrap<T1> U(in.m);
   const Mat<eT>& X = U.M;
@@ -44,14 +44,14 @@ inline void op_max::apply(Mat<typename T1::elem_type>& out, const Op<T1, op_max>
 template <typename eT>
 inline void op_max::apply_noalias(Mat<eT>& out, const Mat<eT>& X, const uword dim,
                                   const typename arma_not_cx<eT>::result* junk) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   arma_ignore(junk);
 
   const uword X_n_rows = X.n_rows;
   const uword X_n_cols = X.n_cols;
 
   if (dim == 0) {
-    arma_extra_debug_print("op_max::apply(): dim = 0");
+    arma_debug_print("op_max::apply(): dim = 0");
 
     out.set_size((X_n_rows > 0) ? 1 : 0, X_n_cols);
 
@@ -65,7 +65,7 @@ inline void op_max::apply_noalias(Mat<eT>& out, const Mat<eT>& X, const uword di
       out_mem[col] = op_max::direct_max(X.colptr(col), X_n_rows);
     }
   } else if (dim == 1) {
-    arma_extra_debug_print("op_max::apply(): dim = 1");
+    arma_debug_print("op_max::apply(): dim = 1");
 
     out.set_size(X_n_rows, (X_n_cols > 0) ? 1 : 0);
 
@@ -94,14 +94,14 @@ inline void op_max::apply_noalias(Mat<eT>& out, const Mat<eT>& X, const uword di
 template <typename eT>
 inline void op_max::apply_noalias(Mat<eT>& out, const Mat<eT>& X, const uword dim,
                                   const typename arma_cx_only<eT>::result* junk) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   arma_ignore(junk);
 
   const uword X_n_rows = X.n_rows;
   const uword X_n_cols = X.n_cols;
 
   if (dim == 0) {
-    arma_extra_debug_print("op_max::apply(): dim = 0");
+    arma_debug_print("op_max::apply(): dim = 0");
 
     out.set_size((X_n_rows > 0) ? 1 : 0, X_n_cols);
 
@@ -115,7 +115,7 @@ inline void op_max::apply_noalias(Mat<eT>& out, const Mat<eT>& X, const uword di
       out_mem[col] = op_max::direct_max(X.colptr(col), X_n_rows);
     }
   } else if (dim == 1) {
-    arma_extra_debug_print("op_max::apply(): dim = 1");
+    arma_debug_print("op_max::apply(): dim = 1");
 
     out.set_size(X_n_rows, (X_n_cols > 0) ? 1 : 0);
 
@@ -134,12 +134,12 @@ inline void op_max::apply_noalias(Mat<eT>& out, const Mat<eT>& X, const uword di
 template <typename T1>
 inline void op_max::apply(Cube<typename T1::elem_type>& out,
                           const OpCube<T1, op_max>& in) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
 
   typedef typename T1::elem_type eT;
 
   const uword dim = in.aux_uword_a;
-  arma_debug_check((dim > 2), "max(): parameter 'dim' must be 0 or 1 or 2");
+  arma_conform_check((dim > 2), "max(): parameter 'dim' must be 0 or 1 or 2");
 
   const unwrap_cube<T1> U(in.m);
 
@@ -157,7 +157,7 @@ inline void op_max::apply(Cube<typename T1::elem_type>& out,
 template <typename eT>
 inline void op_max::apply_noalias(Cube<eT>& out, const Cube<eT>& X, const uword dim,
                                   const typename arma_not_cx<eT>::result* junk) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   arma_ignore(junk);
 
   const uword X_n_rows = X.n_rows;
@@ -165,7 +165,7 @@ inline void op_max::apply_noalias(Cube<eT>& out, const Cube<eT>& X, const uword
   const uword X_n_slices = X.n_slices;
 
   if (dim == 0) {
-    arma_extra_debug_print("op_max::apply(): dim = 0");
+    arma_debug_print("op_max::apply(): dim = 0");
 
     out.set_size((X_n_rows > 0) ? 1 : 0, X_n_cols, X_n_slices);
 
@@ -181,7 +181,7 @@ inline void op_max::apply_noalias(Cube<eT>& out, const Cube<eT>& X, const uword
       }
     }
   } else if (dim == 1) {
-    arma_extra_debug_print("op_max::apply(): dim = 1");
+    arma_debug_print("op_max::apply(): dim = 1");
 
     out.set_size(X_n_rows, (X_n_cols > 0) ? 1 : 0, X_n_slices);
 
@@ -207,7 +207,7 @@ inline void op_max::apply_noalias(Cube<eT>& out, const Cube<eT>& X, const uword
       }
     }
   } else if (dim == 2) {
-    arma_extra_debug_print("op_max::apply(): dim = 2");
+    arma_debug_print("op_max::apply(): dim = 2");
 
     out.set_size(X_n_rows, X_n_cols, (X_n_slices > 0) ? 1 : 0);
 
@@ -238,7 +238,7 @@ inline void op_max::apply_noalias(Cube<eT>& out, const Cube<eT>& X, const uword
 template <typename eT>
 inline void op_max::apply_noalias(Cube<eT>& out, const Cube<eT>& X, const uword dim,
                                   const typename arma_cx_only<eT>::result* junk) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   arma_ignore(junk);
 
   const uword X_n_rows = X.n_rows;
@@ -246,7 +246,7 @@ inline void op_max::apply_noalias(Cube<eT>& out, const Cube<eT>& X, const uword
   const uword X_n_slices = X.n_slices;
 
   if (dim == 0) {
-    arma_extra_debug_print("op_max::apply(): dim = 0");
+    arma_debug_print("op_max::apply(): dim = 0");
 
     out.set_size((X_n_rows > 0) ? 1 : 0, X_n_cols, X_n_slices);
 
@@ -262,7 +262,7 @@ inline void op_max::apply_noalias(Cube<eT>& out, const Cube<eT>& X, const uword
       }
     }
   } else if (dim == 1) {
-    arma_extra_debug_print("op_max::apply(): dim = 1");
+    arma_debug_print("op_max::apply(): dim = 1");
 
     out.set_size(X_n_rows, (X_n_cols > 0) ? 1 : 0, X_n_slices);
 
@@ -280,7 +280,7 @@ inline void op_max::apply_noalias(Cube<eT>& out, const Cube<eT>& X, const uword
       }
     }
   } else if (dim == 2) {
-    arma_extra_debug_print("op_max::apply(): dim = 2");
+    arma_debug_print("op_max::apply(): dim = 2");
 
     out.set_size(X_n_rows, X_n_cols, (X_n_slices > 0) ? 1 : 0);
 
@@ -310,7 +310,7 @@ inline void op_max::apply_noalias(Cube<eT>& out, const Cube<eT>& X, const uword
 
 template <typename eT>
 inline eT op_max::direct_max(const eT* const X, const uword n_elem) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
 
   eT max_val_i = priv::most_neg<eT>();
   eT max_val_j = priv::most_neg<eT>();
@@ -342,46 +342,28 @@ inline eT op_max::direct_max(const eT* const X, const uword n_elem) {
 template <typename eT>
 inline eT op_max::direct_max(const eT* const X, const uword n_elem,
                              uword& index_of_max_val) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
 
-  eT max_val_i = priv::most_neg<eT>();
-  eT max_val_j = priv::most_neg<eT>();
+  eT best_val = priv::most_neg<eT>();
+  uword best_index = 0;
 
-  uword best_index_i = 0;
-  uword best_index_j = 0;
+  for (uword i = 0; i < n_elem; ++i) {
+    const eT val = X[i];
 
-  uword i, j;
-  for (i = 0, j = 1; j < n_elem; i += 2, j += 2) {
-    const eT X_i = X[i];
-    const eT X_j = X[j];
-
-    if (X_i > max_val_i) {
-      max_val_i = X_i;
-      best_index_i = i;
-    }
-    if (X_j > max_val_j) {
-      max_val_j = X_j;
-      best_index_j = j;
+    if (val > best_val) {
+      best_val = val;
+      best_index = i;
     }
   }
 
-  if (i < n_elem) {
-    const eT X_i = X[i];
+  index_of_max_val = best_index;
 
-    if (X_i > max_val_i) {
-      max_val_i = X_i;
-      best_index_i = i;
-    }
-  }
-
-  index_of_max_val = (max_val_i > max_val_j) ? best_index_i : best_index_j;
-
-  return (max_val_i > max_val_j) ? max_val_i : max_val_j;
+  return best_val;
 }
 
 template <typename eT>
 inline eT op_max::direct_max(const Mat<eT>& X, const uword row) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
 
   const uword X_n_cols = X.n_cols;
 
@@ -414,10 +396,10 @@ inline eT op_max::direct_max(const Mat<eT>& X, const uword row) {
 
 template <typename eT>
 inline eT op_max::max(const subview<eT>& X) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
 
   if (X.n_elem == 0) {
-    arma_debug_check(true, "max(): object has no elements");
+    arma_conform_check(true, "max(): object has no elements");
 
     return Datum<eT>::nan;
   }
@@ -472,7 +454,7 @@ inline eT op_max::max(const subview<eT>& X) {
 template <typename T1>
 inline typename arma_not_cx<typename T1::elem_type>::result op_max::max(
     const Base<typename T1::elem_type, T1>& X) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
 
   typedef typename T1::elem_type eT;
 
@@ -481,7 +463,7 @@ inline typename arma_not_cx<typename T1::elem_type>::result op_max::max(
   const uword n_elem = P.get_n_elem();
 
   if (n_elem == 0) {
-    arma_debug_check(true, "max(): object has no elements");
+    arma_conform_check(true, "max(): object has no elements");
 
     return Datum<eT>::nan;
   }
@@ -572,7 +554,7 @@ inline typename arma_not_cx<typename T1::elem_type>::result op_max::max(
 template <typename T1>
 inline typename arma_not_cx<typename T1::elem_type>::result op_max::max(
     const BaseCube<typename T1::elem_type, T1>& X) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
 
   typedef typename T1::elem_type eT;
 
@@ -581,7 +563,7 @@ inline typename arma_not_cx<typename T1::elem_type>::result op_max::max(
   const uword n_elem = P.get_n_elem();
 
   if (n_elem == 0) {
-    arma_debug_check(true, "max(): object has no elements");
+    arma_conform_check(true, "max(): object has no elements");
 
     return Datum<eT>::nan;
   }
@@ -641,14 +623,16 @@ inline typename arma_not_cx<typename T1::elem_type>::result op_max::max(
 template <typename T1>
 inline typename arma_not_cx<typename T1::elem_type>::result op_max::max_with_index(
     const Proxy<T1>& P, uword& index_of_max_val) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
 
   typedef typename T1::elem_type eT;
 
   const uword n_elem = P.get_n_elem();
 
   if (n_elem == 0) {
-    arma_debug_check(true, "max(): object has no elements");
+    arma_conform_check(true, "max(): object has no elements");
+
+    index_of_max_val = 0;
 
     return Datum<eT>::nan;
   }
@@ -716,14 +700,16 @@ inline typename arma_not_cx<typename T1::elem_type>::result op_max::max_with_ind
 template <typename T1>
 inline typename arma_not_cx<typename T1::elem_type>::result op_max::max_with_index(
     const ProxyCube<T1>& P, uword& index_of_max_val) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
 
   typedef typename T1::elem_type eT;
 
   const uword n_elem = P.get_n_elem();
 
   if (n_elem == 0) {
-    arma_debug_check(true, "max(): object has no elements");
+    arma_conform_check(true, "max(): object has no elements");
+
+    index_of_max_val = 0;
 
     return Datum<eT>::nan;
   }
@@ -773,7 +759,7 @@ inline typename arma_not_cx<typename T1::elem_type>::result op_max::max_with_ind
 template <typename T>
 inline std::complex<T> op_max::direct_max(const std::complex<T>* const X,
                                           const uword n_elem) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
 
   uword index = 0;
   T max_val = priv::most_neg<T>();
@@ -793,7 +779,7 @@ inline std::complex<T> op_max::direct_max(const std::complex<T>* const X,
 template <typename T>
 inline std::complex<T> op_max::direct_max(const std::complex<T>* const X,
                                           const uword n_elem, uword& index_of_max_val) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
 
   uword index = 0;
   T max_val = priv::most_neg<T>();
@@ -815,7 +801,7 @@ inline std::complex<T> op_max::direct_max(const std::complex<T>* const X,
 template <typename T>
 inline std::complex<T> op_max::direct_max(const Mat<std::complex<T> >& X,
                                           const uword row) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
 
   const uword X_n_cols = X.n_cols;
 
@@ -836,12 +822,12 @@ inline std::complex<T> op_max::direct_max(const Mat<std::complex<T> >& X,
 
 template <typename T>
 inline std::complex<T> op_max::max(const subview<std::complex<T> >& X) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
 
   typedef typename std::complex<T> eT;
 
   if (X.n_elem == 0) {
-    arma_debug_check(true, "max(): object has no elements");
+    arma_conform_check(true, "max(): object has no elements");
 
     return Datum<eT>::nan;
   }
@@ -894,7 +880,7 @@ inline std::complex<T> op_max::max(const subview<std::complex<T> >& X) {
 template <typename T1>
 inline typename arma_cx_only<typename T1::elem_type>::result op_max::max(
     const Base<typename T1::elem_type, T1>& X) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
 
   typedef typename T1::elem_type eT;
   typedef typename get_pod_type<eT>::result T;
@@ -904,7 +890,7 @@ inline typename arma_cx_only<typename T1::elem_type>::result op_max::max(
   const uword n_elem = P.get_n_elem();
 
   if (n_elem == 0) {
-    arma_debug_check(true, "max(): object has no elements");
+    arma_conform_check(true, "max(): object has no elements");
 
     return Datum<eT>::nan;
   }
@@ -965,7 +951,7 @@ inline typename arma_cx_only<typename T1::elem_type>::result op_max::max(
 template <typename T1>
 inline typename arma_cx_only<typename T1::elem_type>::result op_max::max(
     const BaseCube<typename T1::elem_type, T1>& X) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
 
   typedef typename T1::elem_type eT;
   typedef typename get_pod_type<eT>::result T;
@@ -975,7 +961,7 @@ inline typename arma_cx_only<typename T1::elem_type>::result op_max::max(
   const uword n_elem = P.get_n_elem();
 
   if (n_elem == 0) {
-    arma_debug_check(true, "max(): object has no elements");
+    arma_conform_check(true, "max(): object has no elements");
 
     return Datum<eT>::nan;
   }
@@ -1025,7 +1011,7 @@ inline typename arma_cx_only<typename T1::elem_type>::result op_max::max(
 template <typename T1>
 inline typename arma_cx_only<typename T1::elem_type>::result op_max::max_with_index(
     const Proxy<T1>& P, uword& index_of_max_val) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
 
   typedef typename T1::elem_type eT;
   typedef typename get_pod_type<eT>::result T;
@@ -1033,7 +1019,9 @@ inline typename arma_cx_only<typename T1::elem_type>::result op_max::max_with_in
   const uword n_elem = P.get_n_elem();
 
   if (n_elem == 0) {
-    arma_debug_check(true, "max(): object has no elements");
+    arma_conform_check(true, "max(): object has no elements");
+
+    index_of_max_val = 0;
 
     return Datum<eT>::nan;
   }
@@ -1118,7 +1106,7 @@ inline typename arma_cx_only<typename T1::elem_type>::result op_max::max_with_in
 template <typename T1>
 inline typename arma_cx_only<typename T1::elem_type>::result op_max::max_with_index(
     const ProxyCube<T1>& P, uword& index_of_max_val) {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
 
   typedef typename T1::elem_type eT;
   typedef typename get_pod_type<eT>::result T;
@@ -1126,7 +1114,9 @@ inline typename arma_cx_only<typename T1::elem_type>::result op_max::max_with_in
   const uword n_elem = P.get_n_elem();
 
   if (n_elem == 0) {
-    arma_debug_check(true, "max(): object has no elements");
+    arma_conform_check(true, "max(): object has no elements");
+
+    index_of_max_val = 0;
 
     return Datum<eT>::nan;
   }
